import os,re
from Globals import package_home
from Acquisition import Implicit
from OFS.SimpleItem import Item
from Globals import Persistent
from OFS.ObjectManager import ObjectManager
from OFS.PropertyManager import PropertyManager
from OFS.PropertySheets import PropertySheet
from AccessControl.Role import RoleManager
from Globals import InitializeClass

from Products.fileUtilities import getAllDirectoryPageTemplate
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from AccessControl import ClassSecurityInfo
from Products.SecurityUtils.SecurityUtils import setupClassSecurity


comment_re = re.compile(r'\s*#.*')
msg_re = re.compile(r'\s*([^:]*)\s*:\s*([^\s][^\s\s]*)')

def dictModify(d, md) :
	for i in md.keys() :
		if md[i]!=None :
			d[i] = md[i]
	return d

convert_pairs = 	{':':'\\x3A',
	'\n':'\\x0A',
	'\r':'\\x0D',
}

def charEncode(s) :
	r = ''
	v = s.split('\\')
	s = '\\\\'.join(v)
	for i in convert_pairs.keys() :
		v = s.split(i)
		s = convert_pairs[i].join(v)
		print 'i=',i,'iv=',convert_pairs[i],'s=',s
	return s
 
def charDecode(s) :
	for i in convert_pairs.items() :
		v = s.split(i[1])
		s = i[0].join(v)
	v = s.split('\\\\')
	s = '\\'.join(v)
	return s
     
def getTextDictFromLines(lines) :
	d = {}
	for l in lines :
		if comment_re.match(l) :
			continue
		m = msg_re.match(l)
		if m :
			grp = m.groups()
			d[charDecode(grp[0])] = charDecode(grp[1])
	return d

manage_addMiniI18NForm=PageTemplateFile('zpt/manage_addMiniI18N_Form.pt',globals())
def manage_addMiniI18N(p,REQUEST=None) :
	"""
	"""
	d = p
	i = MiniI18N()
	d._setObject(i.id,i)
	
	if REQUEST :
		try:
			url = p.absolute_url()
		except: url=REQUEST['URL1']
		REQUEST.RESPONSE.redirect('%s/manage_main' % url)

def getFirstLang(lang) :
	s = lang.split(',')
	return s[0]

class MiniI18N(Implicit, Persistent,ObjectManager,PropertyManager, RoleManager,Item) :
	"""
	"""
	
	security=ClassSecurityInfo()
	securityDefination = [
        {'perm':'MiniI18N Management','methods':['getCurrentLang','I18nTranslating','setupMiniI18N'],'roles':['Manager'],},
        {'perm':'_Public','methods':['self'],'roles':['Manager']},
	]
	setupClassSecurity(security,securityDefination)

	meta_type = "MiniI18N"
	manage_options = ( \
        {'label': 'Contents', 'action': 'manage_main'}, \
        {'label': 'missedTextList', 'action': 'missedTextList'}, \
        {'label': 'translatedTextList', 'action': 'translatedTextList'}, \
        {'label': 'Translate', 'action': 'translate'}) \
	  + PropertyManager.manage_options \
          + RoleManager.manage_options \
          + Item.manage_options 

	def __init__(self) :
		self.id = 'i18n'
		self.run_mode='dev'
		self.langTextDict = {}
		self.missTextDict = {}
		self.setupMiniI18N()
	
	def change_runmode(self,rmode) :
		self.run_mode=rmode

	def __call__(self,msg,lang=None) :
		"""
		"""
		return self.getLangText(msg,lang)
	
	missedTextList = PageTemplateFile('zpt/I18N/missedTextList.pt', globals())
	setup_i18n = PageTemplateFile('zpt/I18N/setup_i18n.pt', globals())
	translatedTextList = PageTemplateFile('zpt/I18N/translatedTextList.pt', globals())
		
	def getLangText(self,msg,lang=None) :
		"""
		"""
		if not hasattr(self,'run_mode') :
			self.run_mode = 'dev'
		if lang==None :
			lang = self.getCurrentLang(self.REQUEST)
		if lang not in self.langTextDict.keys() :
			self.langTextDict[lang] = {}
		dict = self.langTextDict[lang]
		if msg not in dict.keys() :
			if self.run_mode!='dev':
				return msg
			if not hasattr(self,'missTextDict') :
				self.missTextDict = {}
			if lang not in self.missTextDict.keys() :
				self.missTextDict[lang] = {}
			self.missTextDict[lang][msg] = ''
			return msg
		if self.run_mode!='dev' :
			return dict[msg]
		if not hasattr(self,'missTextDict') :
			self.missTextDict = {}
		if lang not in self.missTextDict.keys() :
			self.missTextDict[lang] = {}
		if msg in self.missTextDict[lang].keys() :
			del self.missTextDict[lang][msg]			
		self._p_changed = 1
		return dict[msg]

	def getMissTextList(self,lang=None) :
		"""
		"""
		if lang==None :
			lang = self.getCurrentLang(self.REQUEST)
		if lang not in self.langTextDict.keys() :
			return ''
		texts = []

		if lang not in self.missTextDict.keys() :
			self.missTextDict[lang] = {}

		keys = self.missTextDict[lang].keys()
		keys.sort()
		for i in keys :
			if i in self.langTextDict[lang].keys() :
				del self.missTextDict[lang][i]
			else :
				texts.append(charEncode(i) + ':' + charEncode(self.missTextDict[lang][i]) )

		self._p_changed = 1
		s =  '\n'.join(texts)
		return s

	def getTranslatedTextList(self,lang=None) :
		"""
		"""
		if lang==None :
			lang = self.getCurrentLang(self.REQUEST)
		if lang not in self.langTextDict.keys() :
			return ''
		texts = []
		keys = self.langTextDict[lang].keys()
		keys.sort()
		for i in keys :
			texts.append(charEncode(i) + ':' + charEncode(self.langTextDict[lang][i]))

		s =  '\n'.join(texts)
		return s

	def I18nTranslating(self,newtexts,lang=None,REQUEST=None) :
		"""
		"""
		if lang==None :
			lang = self.getCurrentLang(self.REQUEST)
		if lang not in self.langTextDict.keys() :
			self.langTextDict[lang] = {}

		textDict = getTextDictFromLines(newtexts.split('\n'))
		d = {}
		if self.langTextDict.has_key(lang) :
			d = self.langTextDict[lang]
		self.langTextDict[lang] = dictModify(d, textDict)
		self._p_changed = 1
		if REQUEST is not None:
			REQUEST.RESPONSE.redirect('manage_main')

	def setupMiniI18N(self,home=None) :
		"""
		"""

		homeP=globals()
		if type(homeP) is not type('') :
			homeP = package_home(homeP)

		if home!=None :
			homeP=home
			
		p = os.path.join(homeP,'i18n')
		langs = []
		
		for f in os.listdir(p) :
			if os.path.isdir(os.path.join(p,f)) :
				langs.append(f)
		for dir in langs :
			p1 = os.path.join(p,dir,'msg.txt')
			if os.path.exists(p1) :
				f = open(p1,'r')
				textDict = getTextDictFromLines(f.readlines())
				d = {}
				if self.langTextDict.has_key(dir) :
					d = self.langTextDict[dir]
				self.langTextDict[dir] = dictModify(d, textDict)
				f.close()
				
		self._p_changed = 1
		
	def getCurrentLang(self,REQUEST=None) :
		"""
		"""
		if self.REQUEST.cookies.has_key('favorite_language') :
			lang = self.REQUEST.cookies['favorite_language']
		else :
			lang = self.REQUEST.HTTP_ACCEPT_LANGUAGE
			lang = getFirstLang(lang)
		return lang

InitializeClass(MiniI18N)
	
